require "./spec/spec_helper"

describe "Attribute" do
  before :all do
    @subject = CDM::Attribute
  end

  describe "when initialized with a string type and no options" do
    context "with no options" do
      before :each do
        @dummy = @subject.new(:string, "lorem_ipsum")
      end

      it "has the name 'lorem_ipsum'" do
        @dummy.name.should eq "lorem_ipsum"
      end

      it "has the type 'String'" do
        @dummy.type.should eq "String"
      end

      it "is syncable" do
        @dummy.syncable.should eq "YES"
      end

      it "is optional" do
        @dummy.optional.should eq "YES"
      end
    end

    context "with options" do
      before do
        @dummy = @subject.new(:string,
                              "lorem_ipsum_dolor_sit_amet",
                              syncable: "NO",
                              optional: "NO"
                             )
      end

      it "has the name 'lorem_ipsum_dolor_sit_amet'" do
        @dummy.name.should eq "lorem_ipsum_dolor_sit_amet"
      end

      it "has the type 'String'" do
        @dummy.type.should eq "String"
      end

      it "is syncable" do
        @dummy.syncable.should eq "NO"
      end

      it "is optional" do
        @dummy.optional.should eq "NO"
      end
    end
  end

  describe "when initialized with a double type" do
    context "with no options" do
      before do
        @dummy = @subject.new(:double, "double_field")
      end

      it "has the name 'double_field'" do
        @dummy.name.should eq "double_field"
      end

      it "has the type 'Double'" do
        @dummy.type.should eq "Double"
      end

      it "is syncable" do
        @dummy.syncable.should eq "YES"
      end

      it "is optional" do
        @dummy.optional.should eq "YES"
      end

      it "has a defaultValueString equal to '0.0'" do
        @dummy.defaultValueString.should eq "0.0"
      end
    end

    context "with options" do
      before do
        @dummy = @subject.new(:double, "dd", defaultValueString: "0.00")
      end

      it "has the name 'dd'" do
        @dummy.name.should eq "dd"
      end

      it "has the type 'Double'" do
        @dummy.type.should eq "Double"
      end

      it "is syncable" do
        @dummy.syncable.should eq "YES"
      end

      it "is optional" do
        @dummy.optional.should eq "YES"
      end

      it "has a defaultValueString equal to '0.0'" do
        @dummy.defaultValueString.should eq "0.00"
      end
    end
  end

  describe "when initialized with an 'Integer 32' type" do
    context "with no options" do
      before do
        @dummy = @subject.new(:int32, "fixnum")
      end

      it "has the name 'fixnum'" do
        @dummy.name.should eq "fixnum"
      end

      it "has the type 'Integer 32'" do
        @dummy.type.should eq "Integer 32"
      end

      it "is syncable" do
        @dummy.syncable.should eq "YES"
      end

      it "is optional" do
        @dummy.optional.should eq "YES"
      end

      it "has a defaultValueString equal to '0'" do
        @dummy.defaultValueString.should eq "0"
      end
    end

    context "with options" do
      before do
        @dummy = @subject.new(:int32,
                              "fn",
                              defaultValueString: "1",
                              syncable: "NO"
                             )
      end

      it "has the name 'fn'" do
        @dummy.name.should eq "fn"
      end

      it "has the type 'Integer 32'" do
        @dummy.type.should eq "Integer 32"
      end

      it "is not syncable" do
        @dummy.syncable.should eq "NO"
      end

      it "is optional" do
        @dummy.optional.should eq "YES"
      end

      it "has a defaultValueString equal to '1'" do
        @dummy.defaultValueString.should eq "1"
      end
    end
  end

  describe "when initialized with a 'Transformable' type" do
    context "with no options" do
      before do
        @dummy = @subject.new(:transformable, "trans")
      end

      it "has the name 'trans'" do
        @dummy.name.should eq "trans"
      end

      it "has the type 'Transformable'" do
        @dummy.type.should eq "Transformable"
      end

      it "is syncable" do
        @dummy.syncable.should eq "YES"
      end

      it "is optional" do
        @dummy.optional.should eq "YES"
      end
    end

    context "with options" do
      before do
        @dummy = @subject.new(:transformable,
                              "transform",
                              syncable: "NO"
                             )
      end

      it "has the name 'transform'" do
        @dummy.name.should eq "transform"
      end

      it "has the type 'Transformable'" do
        @dummy.type.should eq "Transformable"
      end

      it "is not syncable" do
        @dummy.syncable.should eq "NO"
      end

      it "is optional" do
        @dummy.optional.should eq "YES"
      end
    end
  end

  describe ".xml_attributes" do
    context "when initialized with a string" do
      context "with no options" do

        before do
          @serializable = @subject.new(:string, :string)
        end

        it "returns 4 attributes" do
          @serializable.xml_attributes.keys.length.should eq 4
        end

      end
    end
  end

  describe ".xml_attributes_as_string" do

    context "when adding a string" do
      before do
        @attribute = @subject.new(:string, :attribute)
      end

      it "returns a string with one xml node attribute" do
        expected = 'type="String" name="attribute" syncable="YES" optional="YES"'
        @attribute.xml_attributes_as_string.should eq(expected)
      end
    end

    context "when adding an integer" do
      before do
        @attribute = @subject.new(:int32, :attribute)
      end

      it "returns a string with one xml node attribute" do
        expected = 'type="Integer 32" name="attribute" syncable="YES" optional="YES" defaultValueString="0"'
        @attribute.xml_attributes_as_string.should eq(expected)
      end
    end
  end
end
